#include "vorbis.h"

#define READ 1024
//signed char readbuffer[READ*4*44]; /* out of the data segment, not the stack */


VorbisEncode::VorbisEncode()
{
  ogg_stream_state os; 
  ogg_page         og; 
  ogg_packet       op; 
  vorbis_info      vi; 			
  vorbis_comment   vc; 
  vorbis_dsp_state vd; 
  vorbis_block     vb; 
  int eos=0;
  FILE *of;
  shout_conn_t conn;
}


void VorbisEncode::ShoutInit( char *hostip, 
			      int port, 
			      char *pass, 
			      char *mount) 
{
  shout_init_connection(&conn);
  conn.ip = hostip;
  conn.port = port;
  conn.password = pass;
  conn.mount = mount;
}


void VorbisEncode::SetupEncode( unsigned int channels,
			        unsigned int bitrate,
				unsigned int samplerate,
				FILE *file,
				const char *filename) 
{
  /* choose an encoding mode */
  vorbis_info_init(&vi);
  vorbis_encode_init(&vi,channels,samplerate, -1, bitrate, -1);

  /* add a comment */
  vorbis_comment_init(&vc);
  //vorbis_comment_add(&vc,"url=http://kunstradio.at");

  /* set up the analysis state and auxiliary encoding storage */
  vorbis_analysis_init(&vd,&vi);
  vorbis_block_init(&vd,&vb);
  
  /* set up our packet->stream encoder */
  /* pick a random serial number; that way we can more likely build
     chained streams just by concatenation */
  srand(time(NULL));
  ogg_stream_init(&os,rand());

  if ( (of = fopen(filename, "w")) == NULL) {
    fprintf(stderr, "couldn't open vorbis outfile for writing");
    exit(1);
  }
}

int VorbisEncode::Connect() 
{
  int tmp;

  if (shout_connect(&conn)) {
    printf("Connected to server...\n");
  } else { return -1; }

  /* Vorbis streams begin with three headers; the initial header (with
     most of the codec setup parameters) which is mandated by the Ogg
     bitstream spec.  The second header holds any comment fields.  The
     third header holds the bitstream codebook.  We merely need to
     make the headers, then pass them to libvorbis one at a time;
     libvorbis handles the additional Ogg bitstream constraints */

  {
    ogg_packet header;
    ogg_packet header_comm;
    ogg_packet header_code;

    vorbis_analysis_headerout(&vd,&vc,&header,&header_comm,&header_code);
    ogg_stream_packetin(&os,&header); /* automatically placed in its own
					 page */
    ogg_stream_packetin(&os,&header_comm);
    ogg_stream_packetin(&os,&header_code);
    
	/* We don't have to write out here, but doing so makes streaming 
	 * much easier, so we do, flushing ALL pages. This ensures the actual
	 * audio data will start on a new page
	 */
	while(!eos){
		int result=ogg_stream_flush(&os,&og);
		if(result==0)break;
		tmp = shout_send_data(&conn, og.header, og.header_len);
		if (!tmp) 
		  printf("DEBUG: Send error: %i...\n", conn.error);
		tmp = shout_send_data(&conn, og.body, og.body_len);
		if (!tmp) 
		  printf("DEBUG: Send error: %i...\n", conn.error);
		shout_sleep(&conn);
		fwrite(og.header,1,og.header_len,of);
		fwrite(og.body,1,og.body_len,of);
	}
	
  }
  return 0;
}

void VorbisEncode::Encode(char *readbuffer, long bytes, int channels)
{

  long i; int j; int tmp;
  
  /* data to encode */
  float **buffer=vorbis_analysis_buffer(&vd,READ);
  
  /* expose the buffer to submit data */
  /* uninterleave samples */
  
  for(i = 0; i < bytes/(channels*2); i++)
    {
      for(j=0; j < channels; j++)
	{
	  buffer[j][i] = ((readbuffer[i*2*channels + 2*j + 1]<<8) |
			  (readbuffer[i*2*channels + 2*j] & 0xff))/32768.0f;
	}
    }
  
  /* tell the library how much we actually submitted */
  vorbis_analysis_wrote(&vd,i);
  
  
  /* vorbis does some data preanalysis, then divvies up blocks for
     more involved (potentially parallel) processing.  Get a single
     block for encoding now */
  while(vorbis_analysis_blockout(&vd,&vb)==1){

    /* analysis */
    vorbis_analysis(&vb,&op);
      
    /* weld the packet into the bitstream */
    ogg_stream_packetin(&os,&op);

    /* write out pages (if any) */
    while(!eos){
      int result=ogg_stream_pageout(&os,&og);
      if(result==0)break;

      tmp = shout_send_data(&conn, og.header, og.header_len);
      if (!tmp) 
	printf("shout: Send error: %i...\n", conn.error);
      tmp = shout_send_data(&conn, og.body, og.body_len);
      if (!i) 
	printf("shout: Send error: %i...\n", conn.error);
      
      shout_sleep(&conn);
      
      fwrite(og.header,1,og.header_len, of);
      fwrite(og.body,1,og.body_len, of);
      
      /* this could be set above, but for illustrative purposes, I do
	 it here (to show that vorbis does know where the stream ends) */
      
      if(ogg_page_eos(&og))eos=1;

    }
  }
}


void VorbisEncode::CleanUp()
{
  /* clean up and exit.  vorbis_info_clear() must be called last */ 
  vorbis_analysis_wrote(&vd,0);  //this was the breaker in the encode loop

  ogg_stream_clear(&os);
  vorbis_block_clear(&vb);
  vorbis_dsp_clear(&vd);
  vorbis_comment_clear(&vc);
  vorbis_info_clear(&vi);
  fclose(of);

}
